@@ -893,7 +893,11 @@ XInputWireToCookie(
             }
             return ENQUEUE_EVENT;
 
-        case XI_RawEvent:
+        case XI_RawKeyPress:
+        case XI_RawKeyRelease:
+        case XI_RawButtonPress:
+        case XI_RawButtonRelease:
+        case XI_RawMotion:
             *cookie = *(XGenericEventCookie*)save;
             if (!wireToRawEvent((xXIRawEvent*)event, cookie))
             {
@@ -1229,8 +1233,8 @@ XInputCopyCookie(Display *dpy, XGenericEventCookie *in, XGenericEventCookie *out
     out->data = NULL;
     out->cookie = 0;
 
-    /**/
-    switch(in->evtype) { case XI_Motion:
+    switch(in->evtype) {
+        case XI_Motion:
         case XI_ButtonPress:
         case XI_ButtonRelease:
         case XI_KeyPress:
@@ -1252,7 +1256,11 @@ XInputCopyCookie(Display *dpy, XGenericEventCookie *in, XGenericEventCookie *out
         case XI_PropertyEvent:
             ret = copyPropertyEvent(in, out);
             break;
-        case XI_RawEvent:
+        case XI_RawKeyPress:
+        case XI_RawKeyRelease:
+        case XI_RawButtonPress:
+        case XI_RawButtonRelease:
+        case XI_RawMotion:
             ret = copyRawEvent(in, out);
             break;
         default:
@@ -1295,6 +1303,7 @@ wireToDeviceEvent(xXIDeviceEvent *in, XGenericEventCookie* cookie)
     out->root_y = FP1616toDBL(in->root_y);
     out->event_x = FP1616toDBL(in->event_x);
     out->event_y = FP1616toDBL(in->event_y);
+    out->flags = in->flags;
     out->mods.base = in->mods.base_mods;
     out->mods.locked = in->mods.locked_mods;
     out->mods.latched = in->mods.latched_mods;
@@ -1552,7 +1561,7 @@ wireToRawEvent(xXIRawEvent *in, XGenericEventCookie *cookie)
     out->time           = in->time;
     out->detail         = in->detail;
     out->deviceid       = in->deviceid;
-    out->eventtype      = in->eventtype;
+    out->flags          = in->flags;
 
     out->valuators.mask_len = in->valuators_len * 4;
     out->valuators.mask = next_block(&ptr, out->valuators.mask_len);
@@ -1565,7 +1574,9 @@ wireToRawEvent(xXIRawEvent *in, XGenericEventCookie *cookie)
     for (i = 0; i < bits; i++)
     {
         out->valuators.values[i] = values->integral;
-        out->raw_values[i] = (values + len)->integral;
+        out->valuators.values[i] += ((double)values->frac / (1 << 16) / (1 << 16));
+        out->raw_values[i] = (values + bits)->integral;
+        out->valuators.values[i] += ((double)(values + bits)->frac / (1 << 16) / (1 << 16));
         values++;
     }
 
